# frozen_string_literal: true

require 'apple_core/action_controller/resource_naming'
require 'apiphobic/authorization/transformers/json_api_to_rails_attributes'
require 'apiphobic/resource/collection'
require 'apiphobic/resource/model'

module  Apiphobic
module  Authorization
module  Resource
  RESOURCE_COLLECTION_ACTIONS = %w{index}.freeze

  module ClassMethods
    def authorizer_class
      @authorizer_class ||= Object.const_get(authorizer_class_name)
    end

    def authorizer_parameters_class
      @authorizer_parameters_class ||= \
        Object.const_get(authorizer_class_name('Parameters'))
    end

    def authorizer_scope_class
      @authorizer_scope_class ||= Object.const_get(authorizer_class_name('Scope'))
    end

    def authorized_scope_root_class
      @authorized_scope_root_class ||= Object.const_get(singular_resource_class_name)
    end

    def authorizer_class_components(type = nil)
      [
        name_components['root_module'],
        'Authorizers',
        resource_name,
        type,
      ]
        .compact
    end

    def authorizer_class_name(type = nil)
      authorizer_class_components(type).join('::')
    end
  end

  def self.included(base)
    base.include AppleCore::ActionController::ResourceNaming
    base.extend  ClassMethods

    base.before_action :authorize
  end

  private

  def authorize
    return if authorizer.public_send(authorization_query)

    Erratum.fail(
      'Forbidden',
      resource_name: self.class.singular_underscored_base_resource_name,
      resource_id:   [params[:id]],
      action:        action_name,
    )
  end

  def authorized_parameters
    @authorized_parameters ||= self
                                 .class
                                 .authorizer_parameters_class
                                 .new(action:     action_name,
                                      audience:   authorized_audience,
                                      issuer:     authorized_issuer,
                                      parameters: params,
                                      token:      token)
                                 .call
  end

  def authorized_inclusions
    @authorized_inclusions ||= authorized_parameters[:include]
  end

  def authorized_scope
    @authorized_scope ||= self
                            .class
                            .authorizer_scope_class
                            .new(action:     action_name,
                                 audience:   authorized_audience,
                                 issuer:     authorized_issuer,
                                 parameters: authorized_parameters,
                                 scope_root: self.class.authorized_scope_root_class,
                                 token:      token)
                            .call
  end

  def authorized_attributes
    @authorized_attributes ||= Authorization::Transformers::JsonApiToRailsAttributes
                                 .new(parameters: authorized_parameters.slice(:data))
                                 .call
  end

  def authorizer
    @authorizer ||= self
                      .class
                      .authorizer_class
                      .new(action:     action_name,
                           audience:   authorized_audience,
                           issuer:     authorized_issuer,
                           parameters: authorized_parameters,
                           resource:   authorized_resource&.processed,
                           token:      token)
  end

  def authorized_resource
    return if RESOURCE_COLLECTION_ACTIONS.include?(action_name)

    @authorized_resource ||= \
      ::Apiphobic::Resource::Model
        .new(resource:   Object.const_get(self.class.singular_resource_class_name),
             id:         params[:id],
             parameters: authorized_parameters)
  end

  def authorized_collection
    return unless RESOURCE_COLLECTION_ACTIONS.include?(action_name)

    @authorized_collection ||= \
      ::Apiphobic::Resource::Collection
        .new(resource:   authorized_scope,
             parameters: authorized_parameters)
  end

  def authorized_audience
    current_audience
  end

  def authorized_issuer
    current_issuer
  end

  def authorization_query
    @authorization_query ||= "able_to_#{action_name}?"
  end
end
end
end
