# frozen_string_literal: true

require 'apple_core/refinements/string'

module  Apiphobic
module  JsonApi
class   Relationship
  using ::AppleCore::Refinements::String

  attr_accessor :name,
                :raw_relationship,
                :data_class

  def initialize(name:, data:, data_class: nil)
    self.name             = name
    self.raw_relationship = data
    self.data_class       = data_class || data.class
  end

  def to_rails_attributes
    __send__("#{embedded}_#{type}_to_rails_attributes")
  end

  private

  def embedded_has_one_to_rails_attributes
    attribute = "#{name.singularize}_attributes".to_sym

    with_data_class(attribute, relationship_attributes_to_rails_attributes(data))
  end

  def referenced_has_one_to_rails_attributes
    attribute = "#{name.singularize}_id".to_sym

    with_data_class(attribute, data[:id])
  end

  def embedded_has_many_to_rails_attributes
    attribute = "#{name.pluralize}_attributes".to_sym

    has_many_attributes = data.map do |datum|
      relationship_attributes_to_rails_attributes(datum)
    end

    with_data_class(attribute, has_many_attributes)
  end

  def referenced_has_many_to_rails_attributes
    attribute = "#{name.singularize}_ids".to_sym

    has_many_attributes = data.map { |d| d[:id] }

    with_data_class(attribute, has_many_attributes)
  end

  def empty_empty_to_rails_attributes
    attribute = name.to_sym

    with_data_class(attribute, nil)
  end

  def empty_has_many_to_rails_attributes
    attribute = "#{name.singularize}_ids".to_sym

    with_data_class(attribute, [])
  end

  def relationship_attributes_to_rails_attributes(other)
    other[:attributes].dup.tap do |attrs|
      attrs.delete(:__id__)
      attrs[:id] = other[:id] if other[:id]
    end
  end

  def type
    return 'has_many' if     has_many?
    return 'has_one'  if     has_one?
    return 'empty'    unless has_data?
  end

  def embedded
    if referenced?
      'referenced'
    elsif embedded?
      'embedded'
    elsif !has_data?
      'empty'
    end
  end

  def has_many?
    data.is_a?(::Array)
  end

  def has_one?
    data.is_a?(::Hash) || data.is_a?(::ActionController::Parameters)
  rescue NameError
    false
  end

  def referenced?
    return false unless has_data?

    !Array(data)[0][:attributes]
  end

  def embedded?
    return false unless has_data?

    Array(data)[0][:attributes]
  end

  def has_data?
    @has_data ||= Array(data).compact.any?
  end

  def data
    raw_relationship[:data]
  end

  def with_data_class(key, value)
    if data_class == ::Hash
      { key => value }
    elsif data_class == ::ActionController::Parameters
      ::ActionController::Parameters.new(key => value).tap(&:permit!)
    end
  rescue NameError
    { key => value }
  end
end
end
end
